@@ -1,36 +1,39 @@
-use super::chdman::*;
+use super::chdman;
 use super::checksum::*;
 use super::database::*;
-use super::maxcso::*;
+use super::dolphin;
+use super::maxcso;
 use super::model::*;
 use super::progress::*;
 use super::prompt::*;
-use super::sevenzip::*;
+use super::sevenzip;
 use super::util::*;
 use super::SimpleResult;
 use async_std::path::Path;
-use clap::{App, Arg, ArgMatches, SubCommand};
+use clap::{App, Arg, ArgMatches};
 use indicatif::ProgressBar;
 use rayon::prelude::*;
 use sqlx::sqlite::SqliteConnection;
 use std::collections::HashSet;
+use std::ffi::OsString;
 use std::str::FromStr;
 
-pub fn subcommand<'a, 'b>() -> App<'a, 'b> {
-    SubCommand::with_name("import-roms")
-        .about("Validates and imports ROM files into oxyromon")
+pub fn subcommand<'a>() -> App<'a> {
+    App::new("import-roms")
+        .about("Validate and import ROM files into oxyromon")
         .arg(
-            Arg::with_name("ROMS")
-                .help("Sets the ROM files to import")
+            Arg::new("ROMS")
+                .help("Set the ROM files to import")
                 .required(true)
-                .multiple(true)
-                .index(1),
+                .multiple_values(true)
+                .index(1)
+                .allow_invalid_utf8(true),
         )
         .arg(
-            Arg::with_name("SYSTEM")
-                .short("s")
+            Arg::new("SYSTEM")
+                .short('s')
                 .long("system")
-                .help("Sets the system number to use")
+                .help("Set the system number to use")
                 .required(false)
                 .takes_value(true),
         )
@@ -38,7 +41,7 @@ pub fn subcommand<'a, 'b>() -> App<'a, 'b> {
 
 pub async fn main(
     connection: &mut SqliteConnection,
-    matches: &ArgMatches<'_>,
+    matches: &ArgMatches,
     progress_bar: &ProgressBar,
 ) -> SimpleResult<()> {
     let romfile_paths: Vec<String> = matches.values_of_lossy("ROMS").unwrap();
@@ -54,7 +57,7 @@ pub async fn main(
     for romfile_path in romfile_paths {
         progress_bar.println(&format!("Processing \"{}\"", &romfile_path));
         let romfile_path = get_canonicalized_path(&romfile_path).await?;
-        import_rom(connection, &progress_bar, &system, &header, &romfile_path).await?;
+        import_rom(connection, progress_bar, &system, &header, &romfile_path).await?;
         progress_bar.println("");
     }
 
@@ -92,20 +95,20 @@ pub async fn import_rom<P: AsRef<Path>>(
     let romfile_extension = romfile_path
         .as_ref()
         .extension()
-        .unwrap()
+        .unwrap_or(&OsString::new())
         .to_str()
         .unwrap()
         .to_lowercase();
 
-    let system_directory = get_system_directory(&mut transaction, progress_bar, &system).await?;
+    let system_directory = get_system_directory(&mut transaction, progress_bar, system).await?;
 
     if ARCHIVE_EXTENSIONS.contains(&romfile_extension.as_str()) {
         import_archive(
             &mut transaction,
-            &progress_bar,
+            progress_bar,
             &system_directory,
-            &system,
-            &header,
+            system,
+            header,
             &romfile_path,
             &romfile_extension,
         )
@@ -113,30 +116,40 @@ pub async fn import_rom<P: AsRef<Path>>(
     } else if CHD_EXTENSION == romfile_extension {
         import_chd(
             &mut transaction,
-            &progress_bar,
+            progress_bar,
             &system_directory,
-            &system,
-            &header,
+            system,
+            header,
             &romfile_path,
         )
         .await?;
     } else if CSO_EXTENSION == romfile_extension {
         import_cso(
             &mut transaction,
-            &progress_bar,
+            progress_bar,
             &system_directory,
-            &system,
-            &header,
+            system,
+            header,
+            &romfile_path,
+        )
+        .await?;
+    } else if RVZ_EXTENSION == romfile_extension {
+        import_rvz(
+            &mut transaction,
+            progress_bar,
+            &system_directory,
+            system,
+            header,
             &romfile_path,
         )
         .await?;
     } else {
         import_other(
             &mut transaction,
-            &progress_bar,
+            progress_bar,
             &system_directory,
-            &system,
-            &header,
+            system,
+            header,
             &romfile_path,
         )
         .await?;
@@ -157,9 +170,9 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
     romfile_extension: &str,
 ) -> SimpleResult<()> {
     let tmp_directory = create_tmp_directory(connection).await?;
-    let sevenzip_infos = parse_archive(progress_bar, romfile_path)?;
+    let sevenzip_infos = sevenzip::parse_archive(progress_bar, romfile_path)?;
 
-    let mut roms_sevenzip_infos: Vec<(Rom, &ArchiveInfo)> = Vec::new();
+    let mut roms_sevenzip_infos: Vec<(Rom, &sevenzip::ArchiveInfo)> = Vec::new();
     let mut game_ids: HashSet<i64> = HashSet::new();
 
     for sevenzip_info in &sevenzip_infos {
@@ -170,7 +183,7 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
 
         // system has a header or crc is absent
         if header.is_some() || sevenzip_info.crc.is_empty() {
-            let extracted_path = extract_files_from_archive(
+            let extracted_path = sevenzip::extract_files_from_archive(
                 progress_bar,
                 romfile_path,
                 &[&sevenzip_info.path],
@@ -178,7 +191,8 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
             )?
             .remove(0);
             let size_crc =
-                get_file_size_and_crc(progress_bar, &extracted_path, &header, 1, 1).await?;
+                get_file_size_and_crc(connection, progress_bar, &extracted_path, header, 1, 1)
+                    .await?;
             remove_file(progress_bar, &extracted_path, true).await?;
             size = size_crc.0;
             crc = size_crc.1;
@@ -187,7 +201,7 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
             crc = sevenzip_info.crc.clone();
         }
 
-        match find_rom(connection, size, &crc, &system, &progress_bar).await? {
+        match find_rom(connection, size, &crc, system, progress_bar).await? {
             Some(rom) => {
                 game_ids.insert(rom.game_id);
                 roms_sevenzip_infos.push((rom, sevenzip_info));
@@ -203,7 +217,7 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
     // archive contains a single full game with no invalid file
     if roms_sevenzip_infos.len() == sevenzip_infos.len() && game_ids.len() == 1 {
         let game_id = game_ids.drain().last().unwrap();
-        let rom_ids: HashSet<i64> = find_roms_by_game_id(connection, game_id)
+        let rom_ids: HashSet<i64> = find_roms_by_game_id_no_parents(connection, game_id)
             .await
             .into_par_iter()
             .map(|rom| rom.id)
@@ -221,7 +235,7 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
             let game = find_game_by_id(connection, game_id).await;
             for (rom, sevenzip_info) in &roms_sevenzip_infos {
                 if sevenzip_info.path != rom.name {
-                    rename_file_in_archive(
+                    sevenzip::rename_file_in_archive(
                         progress_bar,
                         romfile_path,
                         &sevenzip_info.path,
@@ -232,7 +246,7 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
 
             let new_path = system_directory.as_ref().join(format!(
                 "{}.{}",
-                if roms_sevenzip_infos.len() == 1 {
+                if roms_sevenzip_infos.len() == 1 && !system.arcade {
                     &roms_sevenzip_infos.get(0).unwrap().0.name
                 } else {
                     &game.name
@@ -260,15 +274,28 @@ async fn import_archive<P: AsRef<Path>, Q: AsRef<Path>>(
 
     // all other cases
     for (rom, sevenzip_info) in roms_sevenzip_infos {
-        let extracted_path = extract_files_from_archive(
-            &progress_bar,
+        let extracted_path = sevenzip::extract_files_from_archive(
+            progress_bar,
             romfile_path,
             &[&sevenzip_info.path],
             &tmp_directory.path(),
         )?
         .remove(0);
 
-        let new_path = system_directory.as_ref().join(&rom.name);
+        // put arcade roms in subdirectories as their names aren't unique
+        let new_path = match system.arcade {
+            true => {
+                let game = find_game_by_id(connection, rom.game_id).await;
+                create_directory(
+                    progress_bar,
+                    &system_directory.as_ref().join(&game.name),
+                    true,
+                )
+                .await?;
+                system_directory.as_ref().join(&game.name).join(&rom.name)
+            }
+            false => system_directory.as_ref().join(&rom.name),
+        };
 
         // move file
         copy_file(progress_bar, &extracted_path, &new_path, false).await?;
@@ -295,8 +322,9 @@ async fn import_chd<P: AsRef<Path>, Q: AsRef<Path>>(
 
     if cue_path.is_file().await {
         progress_bar.println("CUE file found, using multiple tracks mode");
-        let (size, crc) = get_file_size_and_crc(progress_bar, &cue_path, &header, 1, 1).await?;
-        let cue_rom = match find_rom(connection, size, &crc, &system, &progress_bar).await? {
+        let (size, crc) =
+            get_file_size_and_crc(connection, progress_bar, &cue_path, header, 1, 1).await?;
+        let cue_rom = match find_rom(connection, size, &crc, system, progress_bar).await? {
             Some(rom) => rom,
             None => {
                 move_to_trash(connection, progress_bar, system, &cue_path).await?;
@@ -304,7 +332,7 @@ async fn import_chd<P: AsRef<Path>, Q: AsRef<Path>>(
             }
         };
 
-        let roms: Vec<Rom> = find_roms_by_game_id(connection, cue_rom.game_id)
+        let roms: Vec<Rom> = find_roms_by_game_id_no_parents(connection, cue_rom.game_id)
             .await
             .into_iter()
             .filter(|rom| rom.id != cue_rom.id)
@@ -314,7 +342,7 @@ async fn import_chd<P: AsRef<Path>, Q: AsRef<Path>>(
             .iter()
             .map(|rom| (rom.name.as_str(), rom.size as u64))
             .collect();
-        let bin_paths = extract_chd_to_multiple_tracks(
+        let bin_paths = chdman::extract_chd_to_multiple_tracks(
             progress_bar,
             romfile_path,
             &tmp_directory.path(),
@@ -324,8 +352,15 @@ async fn import_chd<P: AsRef<Path>, Q: AsRef<Path>>(
         .await?;
         let mut crcs: Vec<String> = Vec::new();
         for (i, bin_path) in bin_paths.iter().enumerate() {
-            let (_, crc) =
-                get_file_size_and_crc(progress_bar, &bin_path, &header, i, bin_paths.len()).await?;
+            let (_, crc) = get_file_size_and_crc(
+                connection,
+                progress_bar,
+                &bin_path,
+                header,
+                i,
+                bin_paths.len(),
+            )
+            .await?;
             crcs.push(crc);
             remove_file(progress_bar, &bin_path, true).await?;
         }
@@ -352,10 +387,11 @@ async fn import_chd<P: AsRef<Path>, Q: AsRef<Path>>(
     } else {
         progress_bar.println("CUE file not found, using single track mode");
         let bin_path =
-            extract_chd_to_single_track(progress_bar, romfile_path, &tmp_directory.path()).await?;
-        let (size, crc) = get_file_size_and_crc(progress_bar, &bin_path, &header, 1, 1).await?;
+            chdman::extract_chd_to_single_track(progress_bar, romfile_path, &tmp_directory.path()).await?;
+        let (size, crc) =
+            get_file_size_and_crc(connection, progress_bar, &bin_path, header, 1, 1).await?;
         remove_file(progress_bar, &bin_path, true).await?;
-        let rom = match find_rom(connection, size, &crc, &system, &progress_bar).await? {
+        let rom = match find_rom(connection, size, &crc, system, progress_bar).await? {
             Some(rom) => rom,
             None => {
                 move_to_trash(connection, progress_bar, system, romfile_path).await?;
@@ -385,10 +421,11 @@ async fn import_cso<P: AsRef<Path>, Q: AsRef<Path>>(
     romfile_path: &P,
 ) -> SimpleResult<()> {
     let tmp_directory = create_tmp_directory(connection).await?;
-    let iso_path = extract_cso(progress_bar, romfile_path, &tmp_directory.path())?;
-    let (size, crc) = get_file_size_and_crc(progress_bar, &iso_path, &header, 1, 1).await?;
+    let iso_path = maxcso::extract_cso(progress_bar, romfile_path, &tmp_directory.path())?;
+    let (size, crc) =
+        get_file_size_and_crc(connection, progress_bar, &iso_path, header, 1, 1).await?;
     remove_file(progress_bar, &iso_path, true).await?;
-    let rom = match find_rom(connection, size, &crc, &system, &progress_bar).await? {
+    let rom = match find_rom(connection, size, &crc, system, progress_bar).await? {
         Some(rom) => rom,
         None => {
             move_to_trash(connection, progress_bar, system, romfile_path).await?;
@@ -408,6 +445,39 @@ async fn import_cso<P: AsRef<Path>, Q: AsRef<Path>>(
     Ok(())
 }
 
+async fn import_rvz<P: AsRef<Path>, Q: AsRef<Path>>(
+    connection: &mut SqliteConnection,
+    progress_bar: &ProgressBar,
+    system_directory: &Q,
+    system: &System,
+    header: &Option<Header>,
+    romfile_path: &P,
+) -> SimpleResult<()> {
+    let tmp_directory = create_tmp_directory(connection).await?;
+    let iso_path = dolphin::extract_rvz(progress_bar, romfile_path, &tmp_directory.path())?;
+    let (size, crc) =
+        get_file_size_and_crc(connection, progress_bar, &iso_path, header, 1, 1).await?;
+    remove_file(progress_bar, &iso_path, true).await?;
+    let rom = match find_rom(connection, size, &crc, system, progress_bar).await? {
+        Some(rom) => rom,
+        None => {
+            move_to_trash(connection, progress_bar, system, romfile_path).await?;
+            return Ok(());
+        }
+    };
+
+    let mut new_rvz_path = system_directory.as_ref().join(&rom.name);
+    new_rvz_path.set_extension(RVZ_EXTENSION);
+
+    // move RVZ if needed
+    rename_file(progress_bar, romfile_path, &new_rvz_path, false).await?;
+
+    // persist in database
+    create_or_update_romfile(connection, &new_rvz_path, &[rom]).await;
+
+    Ok(())
+}
+
 async fn import_other<P: AsRef<Path>, Q: AsRef<Path>>(
     connection: &mut SqliteConnection,
     progress_bar: &ProgressBar,
@@ -416,8 +486,9 @@ async fn import_other<P: AsRef<Path>, Q: AsRef<Path>>(
     header: &Option<Header>,
     romfile_path: &P,
 ) -> SimpleResult<()> {
-    let (size, crc) = get_file_size_and_crc(progress_bar, romfile_path, &header, 1, 1).await?;
-    let rom = match find_rom(connection, size, &crc, &system, &progress_bar).await? {
+    let (size, crc) =
+        get_file_size_and_crc(connection, progress_bar, romfile_path, header, 1, 1).await?;
+    let rom = match find_rom(connection, size, &crc, system, progress_bar).await? {
         Some(rom) => rom,
         None => {
             move_to_trash(connection, progress_bar, system, romfile_path).await?;
@@ -425,7 +496,20 @@ async fn import_other<P: AsRef<Path>, Q: AsRef<Path>>(
         }
     };
 
-    let new_path = system_directory.as_ref().join(&rom.name);
+    // put arcade roms in subdirectories as their names aren't unique
+    let new_path = match system.arcade {
+        true => {
+            let game = find_game_by_id(connection, rom.game_id).await;
+            create_directory(
+                progress_bar,
+                &system_directory.as_ref().join(&game.name),
+                true,
+            )
+            .await?;
+            system_directory.as_ref().join(&game.name).join(&rom.name)
+        }
+        false => system_directory.as_ref().join(&rom.name),
+    };
 
     // move file if needed
     rename_file(progress_bar, romfile_path, &new_path, false).await?;
@@ -443,8 +527,10 @@ async fn find_rom(
     system: &System,
     progress_bar: &ProgressBar,
 ) -> SimpleResult<Option<Rom>> {
-    let rom: Rom;
-    let mut roms = find_roms_by_size_and_crc_and_system_id(connection, size, crc, system.id).await;
+    let rom: Option<Rom>;
+    let mut roms =
+        find_roms_without_romfile_by_size_and_crc_and_system_id(connection, size, crc, system.id)
+            .await;
 
     // abort if no match
     if roms.is_empty() {
@@ -454,20 +540,27 @@ async fn find_rom(
 
     // let user choose the rom if there are multiple matches
     if roms.len() == 1 {
-        rom = roms.remove(0);
-        progress_bar.println(&format!("Matches \"{}\"", rom.name));
+        rom = Some(roms.remove(0));
+        progress_bar.println(&format!("Matches \"{}\"", rom.as_ref().unwrap().name));
     } else {
-        rom = prompt_for_rom(&mut roms)?;
+        let mut roms_games: Vec<(Rom, Game)> = vec![];
+        for rom in roms {
+            let game = find_game_by_id(connection, rom.game_id).await;
+            roms_games.push((rom, game));
+        }
+        progress_bar.println("Please select the correct ROM:");
+        rom = prompt_for_rom(&mut roms_games)?;
     }
 
     // abort if rom already has a file
-    if rom.romfile_id.is_some() {
-        let romfile = find_romfile_by_id(connection, rom.romfile_id.unwrap()).await;
+    if rom.is_some() && rom.as_ref().unwrap().romfile_id.is_some() {
+        let romfile =
+            find_romfile_by_id(connection, rom.as_ref().unwrap().romfile_id.unwrap()).await;
         progress_bar.println(&format!("Duplicate of \"{}\"", romfile.path));
         return Ok(None);
     }
 
-    Ok(Some(rom))
+    Ok(rom)
 }
 
 async fn create_or_update_romfile<P: AsRef<Path>>(
@@ -707,7 +800,7 @@ mod test {
         assert!(Path::new(&romfile.path).is_file().await);
         assert_eq!(rom.romfile_id, Some(romfile.id));
 
-        let sevenzip_infos = parse_archive(&progress_bar, &romfile.path).unwrap();
+        let sevenzip_infos = sevenzip::parse_archive(&progress_bar, &romfile.path).unwrap();
         assert_eq!(sevenzip_infos.len(), 1);
         assert_eq!(
             sevenzip_infos.get(0).unwrap().path,
