@@ -16,7 +16,7 @@ use async_graphql::{
 use async_std::path::Path;
 use async_std::prelude::FutureExt;
 use async_trait::async_trait;
-use clap::{App, Arg, ArgMatches, SubCommand};
+use clap::{App, Arg, ArgMatches};
 use futures::stream::TryStreamExt;
 use http_types::mime::BYTE_STREAM;
 use http_types::{Mime, StatusCode};
@@ -33,26 +33,26 @@ lazy_static! {
 }
 
 #[derive(RustEmbed)]
-#[folder = "public/"]
+#[folder = "build/"]
 struct Assets;
 
-pub fn subcommand<'a, 'b>() -> App<'a, 'b> {
-    SubCommand::with_name("server")
-        .about("Launches the backend server")
+pub fn subcommand<'a>() -> App<'a> {
+    App::new("server")
+        .about("Launch the backend server")
         .arg(
-            Arg::with_name("ADDRESS")
-                .short("a")
+            Arg::new("ADDRESS")
+                .short('a')
                 .long("address")
-                .help("Specifies the server address")
+                .help("Specify the server address")
                 .required(false)
                 .takes_value(true)
                 .default_value("127.0.0.1"),
         )
         .arg(
-            Arg::with_name("PORT")
-                .short("p")
+            Arg::new("PORT")
+                .short('p')
                 .long("port")
-                .help("Specifies the server port")
+                .help("Specify the server port")
                 .required(false)
                 .takes_value(true)
                 .default_value("8000"),
@@ -188,7 +188,13 @@ impl QueryRoot {
     }
 
     async fn roms(&self, game_id: i64) -> Result<Vec<Rom>> {
-        Ok(find_roms_by_game_id(&mut POOL.get().unwrap().acquire().await.unwrap(), game_id).await)
+        Ok(
+            find_roms_by_game_id_parents(
+                &mut POOL.get().unwrap().acquire().await.unwrap(),
+                game_id,
+            )
+            .await,
+        )
     }
 
     async fn total_original_size(&self, system_id: i64) -> Result<i64> {
@@ -268,16 +274,11 @@ impl QueryRoot {
     }
 }
 
-#[derive(Clone)]
-struct AppState {
-    schema: Schema<QueryRoot, EmptyMutation, EmptySubscription>,
-}
-
 async fn serve_asset(req: tide::Request<()>) -> tide::Result {
     let file_path = req.param("path").unwrap_or("index.html");
     match Assets::get(file_path) {
-        Some(bytes) => {
-            let mime = Mime::sniff(bytes.as_ref())
+        Some(file) => {
+            let mime = Mime::sniff(file.data.as_ref())
                 .or_else(|err| {
                     Mime::from_extension(
                         Path::new(file_path).extension().unwrap().to_str().unwrap(),
@@ -286,7 +287,7 @@ async fn serve_asset(req: tide::Request<()>) -> tide::Result {
                 })
                 .unwrap_or(BYTE_STREAM);
             Ok(tide::Response::builder(StatusCode::Ok)
-                .body(tide::Body::from_bytes(bytes.to_vec()))
+                .body(tide::Body::from_bytes(file.data.to_vec()))
                 .content_type(mime)
                 .build())
         }
@@ -294,13 +295,13 @@ async fn serve_asset(req: tide::Request<()>) -> tide::Result {
     }
 }
 
-pub async fn main(pool: SqlitePool, matches: &ArgMatches<'_>) -> SimpleResult<()> {
+pub async fn main(pool: SqlitePool, matches: &ArgMatches) -> SimpleResult<()> {
     POOL.set(pool).expect("Failed to set database pool");
 
     let schema = Schema::build(QueryRoot, EmptyMutation, EmptySubscription)
-        .data(DataLoader::new(SystemLoader))
-        .data(DataLoader::new(GameLoader))
-        .data(DataLoader::new(RomfileLoader))
+        .data(DataLoader::new(SystemLoader, async_std::task::spawn))
+        .data(DataLoader::new(GameLoader, async_std::task::spawn))
+        .data(DataLoader::new(RomfileLoader, async_std::task::spawn))
         .finish();
 
     let ctrlc = CtrlC::new().expect("Cannot use CTRL-C handler");
@@ -311,8 +312,7 @@ pub async fn main(pool: SqlitePool, matches: &ArgMatches<'_>) -> SimpleResult<()
             app.at("/").get(serve_asset);
             app.at("/*path").get(serve_asset);
 
-            app.at("/graphql")
-                .post(async_graphql_tide::endpoint(schema));
+            app.at("/graphql").post(async_graphql_tide::graphql(schema));
 
             let address = matches.value_of("ADDRESS").unwrap();
             let port = matches.value_of("PORT").unwrap();
